/* Sottoscrittore.java */

import java.util.Random;
import java.util.Vector;

/**
 * @author Fosco
 * 
 * Questa classe implementa il generico consumatore
 */
public class Sottoscrittore extends Thread {
  private Canale canale;

  private int identificatore = -1;

  private final static String EOF = "-1";

  private Vector codaMessaggi = new Vector();

  public Sottoscrittore(Canale c, int id) {
    super("S"+id);
    canale = c;
    identificatore = id;
  }

  public void run() {
    try {
      sleep((int) (Math.random() * 3000) + 2000);

      System.out.println("\nSottoscrittore (S" + identificatore
          + "): inizio e sottoscrizione...");

      canale.addSubscriber(this);

      Random r = new Random();

      int N = 2 + (r.nextInt(5));

      System.out.println("Il sottoscrittore S" + identificatore
          + " e' pronto a ricevere " + N + " messaggi");

      for (int i = 0; i < N; i++) {
        String readMsg = read();
        if( readMsg.equals(EOF) ) break; //passo c.
        System.out.println("S" + identificatore + ": " + readMsg);
      }

      canale.removeSubscriber(this);

      System.out.println("\nS" + identificatore
          + ": de-sottoscrizione e terminazione.");
    } catch (Exception e) {
      System.err.println("Si e' verificato un errore, il seguente: " + e);
      e.printStackTrace(System.err);
    }

  }

  /**
   * Chiamata sospensiva in caso la coda dei messaggi sia vuota. NOTA: come
   * effetto collaterale la read consuma il messaggio dalla coda dei messaggi.
   * 
   * @return il messaggio ricevuto, la stringa "-1" come EOF e null se ci sono
   *         problemi
   */
  private synchronized String read() throws InterruptedException {
    if (codaMessaggi.size() == 0) wait();
    String firstReceivedMessage = (String) codaMessaggi.lastElement();
    codaMessaggi.remove(codaMessaggi.size() - 1);
    return firstReceivedMessage;
  }

  public void push(String msg) {
    codaMessaggi.insertElementAt(msg, 0);
  }

  public synchronized void isMessage() {
    notify();
  }

}